import os
import stat
import cgi
import json
from datetime import datetime
from http.server import BaseHTTPRequestHandler
from jinja2 import Environment, PackageLoader
from urllib.parse import unquote
from .util.utils import format_date_time


def create_server(args):
    class SpriteServer(BaseHTTPRequestHandler):

        format = '%Y/%m/%d %H:%M:%S'
        log_format = '%Y-%m-%d %H:%M:%S.%f'

        def do_GET(self):
            root = os.getcwd()
            path = os.path.join(root, unquote(self.path[1:], 'utf-8'))
            if os.path.isfile(path):
                file = open(path, 'rb')
                buffer = file.read()
                self.send_response(200)
                self.send_header('Content-type', 'application/download')
                self.end_headers()
                self.wfile.write(buffer)
            elif os.path.isdir(path):
                parent = self.__get_parent(path)
                (directories, files) = self.__get_list(path)
                env = Environment(loader=PackageLoader(__package__, '../template'))
                template = env.get_template('list.html')
                page = template.render(
                    parent=parent, directories=directories, files=files)
                self.send_response(200)
                self.send_header('Content-type', 'text/html')
                self.end_headers()
                self.wfile.write(page.encode())
            else:
                env = Environment(loader=PackageLoader(__package__, '../template'))
                template = env.get_template('error.html')
                page = template.render(url=self.path)
                self.send_response(404)
                self.send_header('Content-type', 'text/html')
                self.end_headers()
                self.wfile.write(page.encode())

        def do_POST(self):
            try:
                _, pdict = cgi.parse_header(self.headers.get('Content-type'))
                pdict['boundary'] = bytes(pdict['boundary'], 'utf-8')
                multipart_data = cgi.parse_multipart(self.rfile, pdict)
                file_name, file_data = multipart_data.popitem()
                root = os.getcwd()
                path = os.path.join(root, self.path[1:], file_name)
                file = open(path, 'wb')
                file.write(file_data[0])

                self.send_response(200)
                data = {'message': 'Upload successful.'}
            except Exception:
                self.send_response(500)
                data = {'message': 'Upload failed.'}
            
            self.send_header('Content-type', 'application/json')
            self.end_headers()
            self.wfile.write(json.dumps(data).encode())

        def log_message(self, format: str, *args):
            print('[%s] [\u001b[33m%s\u001b[0m] %s' % (datetime.now().strftime(self.log_format)[:-3], self.command, self.headers.get('user-agent')))
            return

        def __get_parent(self, path):
            properties = os.stat(path)
            permission = stat.filemode(properties.st_mode)
            modification = format_date_time(properties.st_mtime, self.format)
            parent = {
                'permission': permission,
                'modification': modification
            }
            return parent

        def __get_list(self, path):
            directories = []
            files = []
            list = os.listdir(path)
            for item in list:
                item_path = os.path.join(path, item)
                properties = os.stat(item_path)
                permission = stat.filemode(properties.st_mode)
                modification = format_date_time(
                    properties.st_mtime, self.format)
                if os.path.isdir(item_path):
                    directories.append({
                        'permission': permission,
                        'modification': modification,
                        'name': item + '/'
                    })
                elif os.path.isfile(item_path):
                    files.append({
                        'permission': permission,
                        'modification': modification,
                        'size': properties.st_size,
                        'name': item
                    })
            return (directories, files)

    return SpriteServer



from http.server import HTTPServer
from netifaces import interfaces, ifaddresses, AF_INET
from .lib.util.constants import PROJECT_VERSION
from .lib.sprite_server import create_server


if __name__ == '__main__':
    import argparse

    parser = argparse.ArgumentParser()
    parser.add_argument('-v', '--version', action='version', version=PROJECT_VERSION,
                        help='Show version number')
    parser.add_argument('port', action='store',
                        default=6060, type=int,
                        nargs='?',
                        help='Specify alternate port [default: 6060]')
    args = parser.parse_args()
    sprite = create_server(args)
    HOST, PORT = 'localhost', args.port
    with HTTPServer((HOST, PORT), sprite) as httpd:
        try:
            print('\u001b[33mSprite server is running.\nVersion: %s\nAvailable on:\u001b[0m' % PROJECT_VERSION)
            names = interfaces()
            for name in names:
                addresses = ifaddresses(name)
                if AF_INET in addresses.keys():
                    print('\u001b[32m  http://%s:%d\u001b[0m' % (addresses[AF_INET][0]['addr'], PORT))
            httpd.serve_forever()
        except KeyboardInterrupt:
            print('\u001b[33mSprite Server has been terminated.\u001b[0m')
            pass


import setuptools
from sprite.lib.util import constants

with open('README.md', 'r', encoding='utf-8') as fh:
    long_description = fh.read()

setuptools.setup(
    name='sprite.server',
    version=constants.PROJECT_VERSION,
    author=constants.PROJECT_AUTHOR,
    author_email=constants.PROJECT_AUTHOR_EMAIL,
    description=constants.PROJECT_DESCRIPTION,
    long_description=long_description,
    long_description_content_type='text/markdown',
    url='',
    include_package_data=True,
    classifiers=['Programming Language :: Python :: 3',
                 'License :: OSI Approved :: ISC License (ISCL)',
                 'Operating System :: OS Independent'],

    install_requires=['Jinja2==3.1.2',
                'MarkupSafe==2.1.1',
                'netifaces==0.11.0']
)
